--------------------------------------------------------------------------------
-- Copyright  2019								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- For more information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------

local AddonName, NS = ...

local L
local ConfirmDialog, OptionsFrame
local CanCopy, CopyButton
local HookedParents = {}

local abs = math.abs
local upper, strplit, clip = string.upper, string.split, string.trim

local ButtonWidth, ButtonHeight = 96, 20

local function FramePos()
	return "TOPLEFT", "TOPLEFT", OptionsFrame:GetLeft(), -(UIParent:GetHeight() - OptionsFrame:GetTop())
end

local function IsFauxMazzleChar(split, char_realm)
	local Char, Realm = strplit(split, char_realm, 2)
	Char = clip(Char)
	Realm = clip(Realm)
	local Settings = NS:GetSettings("Profiles")
	return Settings[Char.."-"..Realm] ~= nil and true or false
end

local MenuItems = {}
function NS:RegisterModule(id, text, icon, func, tip, alpha, spacer, onload, onshow)
	local Spacing = -10
	local Alpha, Tip, Spacer
	if spacer then
		Spacer = Spacing
	end
	if alpha and type(alpha) == "boolean" then
		Spacer = Spacing
	else
		Alpha = alpha
	end
	if tip then
		if type(tip) == "boolean" then
			Spacer = Spacing
		elseif type(tip) == "number" then
			Alpha = tip
		else
			Tip = tip
		end
	end
	tinsert(MenuItems, { id=id, text=text, icon=icon, func=func, tip=Tip, alpha=Alpha, spacer=Spacer, onload=onload, onshow=onshow })
end
local LastOption
local function OnMenuClick(self)
	LastOption = nil
	local point, topoint, x, y = FramePos()
	for i=1, #MenuItems do
		if MenuItems[i].frame ~= self and MenuItems[i].frame:GetChecked() then
			MenuItems[i].frame:SetChecked(false)
		end
		if MenuItems[i].Options then
			MenuItems[i].Options:Hide()
		end
	end
	if not self then 
		OptionsFrame.Banner:Show()
		return 
	end
	if MenuItems[self.ID].frame:GetChecked() then
		local Container = MenuItems[self.ID].Options
		if Container then
			for k, v in pairs(Container) do
				if type(v) == "table" and v.GetObjectType and v.Init and v.InitFunc and type(v.Init) == "function" and type(v.InitFunc) == "function" then
					v:Init()
				end
			end
		end
		local f = MenuItems[self.ID].func(OptionsFrame)--, MenuItems[self.ID].frame)
		OptionsFrame.Banner:Hide()
		LastOption = self
		if not MenuItems[self.ID].Options then
			MenuItems[self.ID].Options = f
			NS.AddBackground(f)
			f:ClearAllPoints()
			f:SetPoint("TOPLEFT", OptionsFrame.DividerV, "TOPRIGHT")
		end
		OptionsFrame:SetWidth(OptionsFrame.MinWidth + f:GetWidth())
		OptionsFrame.Menu:SetHeight(f:GetHeight())
	else
		OptionsFrame.Banner:Show()
	end
	OptionsFrame:ClearAllPoints()
	OptionsFrame:SetPoint(point, UIParent, topoint, x, y)
	PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
end

NS.AddonFuncs = {}
function NS:RegisterAddonOptions(addon, func)
	NS.AddonFuncs[addon] = func
end
function NS:GetAddonOptions(addon)
	return NS.AddonFuncs[addon]
end
----------------------------------------------------------------------------------------------------------------
--						Common UI Functions					      --
----------------------------------------------------------------------------------------------------------------

function NS.AddBackground(self, texture, alpha)
	self.Background = self:CreateTexture(nil, "BACKGROUND", nil, -1)
	self.Background:SetAllPoints()
	self.Background:SetTexture(texture or "Interface\\BUTTONS\\WHITE8X8")
	if alpha then
		self.Background:SetAlpha(alpha)
	else
		self.Background:SetVertexColor(0.1, 0.1, 0.1, 0.9)
	end
end

local function CheckClick(self)
	self:GetParent():SetChecked(not self:GetParent():GetChecked())
	self:GetParent():GetScript("OnClick")(self:GetParent())
end
local function CheckOnEnter(self)
	local Script = self:GetParent():GetScript("OnEnter")
	if Script then
		Script(self:GetParent())
	end
end

local function CheckOnLeave(self)
	local Script = self:GetParent():GetScript("OnLeave")
	if Script then
		Script(self:GetParent())
	end
end

local function EnableLabel(self, enabled)
	if self.LabelButton then
		self.LabelButton:SetEnabled(enabled)
		self.Label:SetAlpha(enabled and 1 or 0.5)
	end
end

function NS:AddCheckLabel(self, label, point)
	self.LabelButton = true
	self:SetLabel(label, point)
	self.LabelButton:SetHighlightTexture(NS.TexturePath.."OptionButton")
	self.LabelButton:SetScript("OnClick", CheckClick)
	self.LabelButton:SetScript("OnEnter", CheckOnEnter)
	self.LabelButton:SetScript("OnLeave", CheckOnLeave)
	self:HookScript("OnEnable", function(self) EnableLabel(self, true) end)
	self:HookScript("OnDisable", function(self) EnableLabel(self, false) end)
end

function NS:CreateDropList(name, parent, data, index, table, display, value, maxdrops)
	local f = NS.Lib:CreateDropList(name, parent, data, index, table, display, value, maxdrops)
	f.Menu:SetBackdropColor(0.06, 0.06, 0.06, 0.6)
	return f
end

function NS:CreateLabel(parent, name, text, linewidth)
--	local lw = linewidth and 46 + linewidth or 46
	local lw = linewidth and linewidth + 30 or 30
	local f = CreateFrame("Frame", "$parent"..name, parent)

	f.BG = f:CreateTexture(nil, "ARTWORK", nil, 2)
	f.BG:SetAllPoints()

--	f.Text = NS.Lib:CreateFont("Text", f, text, "CENTER", "CENTER")
	f.Text = NS.Lib:CreateFont("Text", f, text, "CENTER", "MIDDLE")
	f.Text:SetDrawLayer("ARTWORK", 3)
	f.Text:SetPoint("CENTER")
	f.Text:SetTextColor(0.7, 0.7, 0.7)
	
	f:SetSize(f.Text:GetWidth()+6, 5)

	f.Line1 = f:CreateTexture(nil, "ARTWORK", nil, 1)
	f.Line1:SetPoint("LEFT", f, "RIGHT")
	f.Line1:SetTexture("Interface\\BUTTONS\\WHITE8X8")
	f.Line1:SetVertexColor(0.3, 0.3, 0.3)
	f.Line1:SetSize(lw, 1)

	f.Line2 = f:CreateTexture(nil, "ARTWORK", nil, 1)
	f.Line2:SetPoint("RIGHT", f, "LEFT")
	f.Line2:SetTexture("Interface\\BUTTONS\\WHITE8X8")
	f.Line2:SetVertexColor(0.3, 0.3, 0.3)
	f.Line2:SetSize(lw, 1)
	function f:SetText(text)
		self.Text:SetText(text)
	end
	return f
end

local function CreateButton(name, parent, template, iconwidth, libfont)
	local f = CreateFrame(template, "$parent"..name, parent)
	f:SetSize(25, 15)
	if template == "CheckButton" then
		f:SetCheckedTexture(NS.TexturePath.."OptionButton")
	elseif template == "Button" then
		f:SetPushedTexture(NS.TexturePath.."OptionButton")
	end
	NS.AddBackground(f, NS.TexturePath.."OptionButton", 0.5)
	f.Icon = f:CreateTexture("$parentIcon", "OVERLAY", nil, 1)
	f.Icon:SetPoint("TOPLEFT", 5, 0)
	f.Icon:SetPoint("BOTTOMRIGHT", f, "BOTTOMLEFT", iconwidth, 0)
--	f.Text = NS.Lib:CreateFont("Text", f, "W", "LEFT", "CENTER", libfont and NS.Lib.Font or NS.Font, libfont and NS.Lib.FontSize or NS.FontSize)
	f.Text = NS.Lib:CreateFont("Text", f, "W", "LEFT", "MIDDLE", libfont and NS.Lib.Font or NS.Font, libfont and NS.Lib.FontSize or NS.FontSize)
	f.Text:SetShadowColor(0, 0, 0, 1)
	f.Text:SetShadowOffset(1, -1)
	f.Text:ClearAllPoints()
	f.Text:SetPoint("LEFT", f.Icon, "RIGHT", 5, 0)
	local h = f:CreateTexture(nil)
	h:SetTexture("Interface\\Common\\bluemenu-main")
	h:SetTexCoord(0.00390625, 0.87890625, 0.75195313, 0.83007813)
	h:SetBlendMode("ADD")
	h:SetPoint("TOPLEFT", -5, 5)
	h:SetPoint("BOTTOMRIGHT", 5, -5)
	f:SetHighlightTexture(h)
	f:SetScript("OnDisable", function(self)
		self.Icon:SetAlpha(0.5)
		self.Text:SetAlpha(0.5)
	end)
	f:SetScript("OnEnable", function(self)
		self.Icon:SetAlpha(1)
		self.Text:SetAlpha(1)
	end)
	function f:SetIconAlpha(alpha)
		f.Icon:SetAlpha(alpha)
	end
	function f:SetFont(font, size)
		self.Text:SetFont(font, size, "")
	end
	function f:SetIcon(icon)
		self.Icon:SetTexture(icon and NS.TexturePath..icon or "")
	end
	function f:SetText(text)
		self.Text:SetText(text)
	end
	function f:SetTooltip(text)
		self.Tooltip = text
		f:HookScript("OnEnter", function(self)
			if not self.Tooltip then return end
			GameTooltip:SetOwner(self, "ANCHOR_LEFT")
--			GameTooltip:SetText(text)
			GameTooltip:SetText(text, nil, nil, nil, nil, true)
		end)
		f:HookScript("OnLeave", function(self)
			if not self.Tooltip then return end
			GameTooltip:Hide()
		end)
		f:HookScript("OnClick", function(self)
			GameTooltip:Hide()
		end)
	end
	return f
end

function NS:CreatePlainButton(name, parent, iconwidth)
	local f = CreateButton(name, parent, "Button", iconwidth)
	f:HookScript("OnClick", function(self)
		if self.PostClick then
			self:PostClick()
			PlaySound(32753)
		end
	end)
	return f
end

function NS:CreateAddonButton(id, parent, template, settings)
	local f = CreateButton(id, parent, "Button", 20)
	return f
end

function NS:CreateUpdateAllButton(name, parent, addon, setting, subsetting)
	local f = NS:CreatePlainButton(name, parent, 25)
	f:SetText(L.Update)
	f:SetIcon("AddonOptionsHighlight")
	f.Icon:SetAlpha(0.5)
	NS.Lib:SetLabel(f, L.GetText(L.UpdateAll, L.Characters), "TOP", 0, 3)
	f.Label:SetTextColor(1, 1, 0)
	f.Addon = addon
	f.Setting = setting
	f.SubSettings = subsetting
	f.PostClick = function(self)
		local value
		if self.SubSettings then
			value = NS:GetSettings("Addons")[self.Addon][self.Setting][self.SubSettings]
		else
			value = NS:GetSettings("Addons")[self.Addon][self.Setting]
		end
		local Profiles = NS:GetSettings("Profiles")
		for k, v in pairs(Profiles) do
			if v.ADDONS and v.ADDONS[self.Addon] and v.ADDONS[self.Addon][self.Setting] then
				if self.SubSettings and v.ADDONS[self.Addon][self.Setting][self.SubSettings] then
					v.ADDONS[self.Addon][self.Setting][self.SubSettings] = value
				else
					v.ADDONS[self.Addon][self.Setting] = value
				end
			end
		end
		if self.UpdateProfiles then
			self.UpdateProfiles(IsFauxMazzleChar)
		end
		self.Icon:SetAlpha(1)
		self.Icon:SetVertexColor(0, 1, 0)
	end
	return f
end

function NS:CreateMenuButton(id, parent, template, settings, iconwidth, libfont)
	local f = CreateButton(settings.id, parent, template, iconwidth, libfont)
	f.Template = template
	f.ID = id
	f:SetIcon(settings.icon)
	f:SetIconAlpha(settings.alpha or 0.75)
	f:SetText(settings.nolocale and settings.text or L[settings.text])
	local Tip = settings.tip and (settings.nolocale and settings.tip or L[settings.tip])
	if Tip then
		f:SetTooltip(Tip)
	end
	if settings.func then
		f:HookScript("OnClick", OnMenuClick)
	end
	return f
end

function NS:SetScrollHighlight(self)
	self:SetNormalTexture(NS.TexturePath.."OptionButton")
	self:GetNormalTexture():SetAlpha(0.5)
	self:SetHighlightTexture("Interface\\Common\\bluemenu-main")
	local T = self:GetHighlightTexture()
	T:SetBlendMode("ADD")
	T:SetTexCoord(0.00390625, 0.87890625, 0.75195313, 0.83007813)
end

function NS:LoadUI(initialising)
	if OptionsFrame then
		OptionsFrame:SetReloadHighlight(initialising)
		OptionsFrame:SetShown(not OptionsFrame:IsShown())
		return
	end
	L = NS.TEXTS
	NS.SharedMediaFonts = {}
--	local LSM = LibStub("LibSharedMedia-3.0")
--	for k, v in pairs(LSM:List("font")) do
--		NS.SharedMediaFonts[k] = { text=v }
--	end
	local Settings = NS:GetCharacterSettings("Config", "UI")
	OptionsFrame = NS.Lib:CreateDragHeader("FauxMazzleOptions", UIParent, "Frame", true)
	local f = OptionsFrame
	f:SetClampedToScreen(true)
	f:SetWidth(120)
	f:SetTitleFont("Fonts\\ARIALN.TTF", 12)
	f:SetTitleColor(0.9, 0.7, 0)
	f:SetTitleFont(NS.Font, 14)
	f:SetTitle(L.Options)
	f:SetFrameStrata("MEDIUM")
	if not initialising then -- Should only work for new installs and new/reset characters
		tinsert(UISpecialFrames, f:GetName())
	end
	f:ClearAllPoints()
--	f:SetPoint(Settings.point or "TOPLEFT", UIParent, Settings.topoint or "TOPLEFT", Settings.x or 80, Settings.y or -60)
	f:SetPoint(Settings.point or "TOPLEFT", UIParent, Settings.topoint or "TOPLEFT", Settings.x or 80, Settings.y or -60)
	f.MiniButtion = NS:GetFrame("Utility", "MiniButton")
	f.Waving = 67
	f.MinMenuHeight = 140
	f.MinOptionsWidth = 60
	f.DividerSize = 3
	f:SetScript("OnDragStop", function(self) 
		self:StopMovingOrSizing() 
--		local p, r, t, x, y = self:GetPoint(1)
		Settings.point, Settings.topoint, Settings.x, Settings.y = FramePos()
	end)
	f:SetScript("OnShow", function(self) 
		PlaySound(SOUNDKIT.UI_IG_STORE_WINDOW_OPEN_BUTTON, "Master")
		self.MiniButtion:SetPortrait(true)
		self.TimeToPlay = 1
	end)
	f:SetScript("OnHide", function(self) 
		self.TimeToPlay = nil
		self.MiniButtion:SetPortrait()
		OnMenuClick()
		if ConfirmDialog then 
			ConfirmDialog:Hide()
		end
		PlaySoundFile(567433, "Master")
--		PlaySoundFile(631886, "Master")
	end)
	f:SetScript("OnUpdate", function(self, elapsed)
		if not self.TimeToPlay then return end
		self.TimeToPlay = self.TimeToPlay - elapsed
		if self.TimeToPlay < 0 then
			self.MiniButtion.Portrait3D:SetAnimation(self.Waving)
			self.TimeToPlay = random(3, 14)
		end
	end)
	function f:SetReloadHighlight(highlight)--highlight) If it needs it why would we hide if before ReloadUI?
		local Reload = MenuItems[#MenuItems].frame
		Reload.Reason:SetText()
		if highlight then
			Reload.Background:SetVertexColor(1, 0, 0)
			Reload.Background:SetAlpha(0.7)
			Reload.Reason:SetText(NS.ReasonText)
		else
			Reload.Background:SetVertexColor(1, 1, 1)
			Reload.Background:SetAlpha(0.9)
		end
	end
	f.Logo = f:CreateTexture()
	local Size = f:GetHeight() * 0.6
	f.Logo:SetSize(Size, Size)
	f.Logo:SetTexture(NS:GetOption("TexturePath").."FM")
	f.Logo:SetPoint("LEFT", 4, 0)
	f.DividerH = f:CreateTexture(nil, "BACKGROUND", nil, 1)
	f.DividerH:SetTexture("Interface\\BUTTONS\\WHITE8X8")
	f.DividerH:SetPoint("TOPLEFT", f, "BOTTOMLEFT")
	f.DividerH:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 0, -f.DividerSize)
	f.Menu = f:CreateTexture()
	f.Menu:SetTexture("Interface\\BUTTONS\\WHITE8X8")
	f.Menu:SetVertexColor(0.1, 0.1, 0.1, 0.9)
	f.Menu:SetSize(120, 40)
	f.Menu:SetPoint("TOPLEFT", f.DividerH, "BOTTOMLEFT")
	f.DividerV = f:CreateTexture()
	f.DividerV:SetTexture("Interface\\BUTTONS\\WHITE8X8")
	f.DividerV:SetPoint("TOPLEFT", f.Menu, "TOPRIGHT")
	f.DividerV:SetPoint("BOTTOMRIGHT", f.Menu, "BOTTOMRIGHT", f.DividerSize, 0)
	f.DividerV.Divider = f.DividerH
	hooksecurefunc(f.DividerV, "SetVertexColor", function(self, r, g, b, a)
		self:GetParent().DividerH:SetVertexColor(r, g, b, a)
		self:GetParent().Banner.Mazzle:SetVertexColor(r, g, b, a, 1)
	end)
	f.MinWidth = f.Menu:GetWidth() + f.DividerV:GetWidth()
	f.Banner = f:CreateTexture()
	f.Banner:SetSize(50, 120)
	f.Banner:SetTexture("Interface\\BUTTONS\\WHITE8X8")
	f.Banner:SetVertexColor(0.1, 0.1, 0.1, 0.9)
	f.Banner:SetPoint("TOPLEFT", f.DividerV, "TOPRIGHT")
	f.Banner:SetPoint("BOTTOMRIGHT", f.DividerV, 50 , 0)
	hooksecurefunc(f.Banner, "Hide", function(self)
		self.Faux:Hide()
		self.Mazzle:Hide()
	end)
	hooksecurefunc(f.Banner, "Show", function(self)
		local parent = self:GetParent()
		parent:SetWidth(parent.MinWidth + parent.Banner:GetWidth())
		parent.Menu:SetHeight(parent.Menu.MinHeight)
		self.Faux:Show()
		self.Mazzle:Show()
	end)
	local layer, level = f.Banner:GetDrawLayer()
	f.Banner.Faux = f:CreateTexture()
	f.Banner.Faux:SetDrawLayer(layer, level + 1)
	f.Banner.Faux:SetSize(30, 60)
	f.Banner.Faux:SetBlendMode("DISABLE")
	f.Banner.Faux:SetTexture(NS.TexturePath.."Faux")
	f.Banner.Faux:SetAlpha(0.98)
	f.Banner.Faux:SetPoint("BOTTOM", f.Banner, "CENTER", 3, 12)
	f.Banner.Mazzle = f:CreateTexture()
	f.Banner.Mazzle:SetDrawLayer(layer, level + 1)
	f.Banner.Mazzle:SetSize(30, 60)
	f.Banner.Mazzle:SetBlendMode("DISABLE")
	f.Banner.Mazzle:SetTexture(NS.TexturePath.."Mazzle")
	f.Banner.Mazzle:SetAlpha(0.98)
	f.Banner.Mazzle:SetPoint("TOP", f.Banner, "CENTER", 3, 10)
	local function ResetOnLoad(self)
		SetPortraitTexture(self.Icon, "player")
		if not self.Icon:GetTexture() then
			C_Timer.After(5, function() SetPortraitTexture(self.Icon, "player") end)
		end
	end
	local function CharacterReset(reset)
		if reset then
			local Settings = NS:GetSettings("Profile")
			Settings = nil
			FauxMazzleConfig(nil, NS:GetOption("SetupType").Character + NS:GetOption("SetupType").Reset)
			NS:DefaultsAddonConfigInitialiseAll(OptionsFrame)
		end
	end
	local function CharacterResetOnClick(parent, self)
		NS:ConfirmAction(parent, self, L.ConfirmReset, CharacterReset)
		return ConfirmDialog
	end
	NS:RegisterModule("Reset", "Reset", nil, CharacterResetOnClick, "ResetTip", 1, true, ResetOnLoad)

	local function UIResetOnLoad(self)
--		self.Reason = NS.Lib:CreateFont("Reason", self, "", "CENTER", "CENTER", "Fonts\\ARIALN.TTF", NS.FontSize-3)
		self.Reason = NS.Lib:CreateFont("Reason", self, "", "CENTER", "MIDDLE", "Fonts\\ARIALN.TTF", NS.FontSize-3)
		self.Reason:SetTextColor(1, 1, 0)
		self.Reason:SetPoint("TOP", self, "BOTTOM", 0, -3)
	end
	local function UIReset(reset, menubutton)
		if reset then
--			if menubutton.Checkbox:GetChecked() then -- Reset all addons etc.
--			end
			C_UI.Reload()
		end
	end
	local function UIResetOnClick(parent, self)
		local f = NS:ConfirmAction(parent, self, L.ConfirmReload, UIReset)
--		if not self.ControlsAdded then
--			self.ControlsAdded = true
--			f:SetButtonPos(self, -35)
--			f:ShowControls(self)
--			self.Checkbox = ...
--			f:AddControl(self, self.Checkbox, -10, -5, "TOP")
--		end
		return ConfirmDialog
	end

	NS:RegisterModule("Reload", "Reload", "Reload", UIResetOnClick, "ReloadTip", 1, true, UIResetOnLoad)

	local TotalHeight = 20
	for i=1, #MenuItems do
		local b = NS:CreateMenuButton(i, f, "CheckButton", MenuItems[i], ButtonHeight+5)
		if MenuItems[i].id == "HUD" then
			b.Icon:SetVertexColor(1, 0, 1)
		end
		MenuItems[i].frame = b
		b:SetSize(ButtonWidth, ButtonHeight)
		if i == 1 then
			b:SetPoint("TOPLEFT", f.Menu, 6, -(TotalHeight*0.5))
		else
			b:SetPoint("TOP", MenuItems[i-1].frame, "BOTTOM", 0, MenuItems[i].spacer or 0)
		end
		if MenuItems[i].onload then
			MenuItems[i].onload(b)
		end
		if MenuItems[i].onshow then
			b:SetScript("OnShow", function(self)
				MenuItems[i].onshow(self)
			end)
			MenuItems[i].onshow(b)
		end
		TotalHeight = TotalHeight + ButtonHeight + (MenuItems[i].spacer and abs(MenuItems[i].spacer) or 0)
	end
	f.Menu.MinHeight = TotalHeight+5

--	MenuItems[#MenuItems].frame:SetHighlightTexture(nil)
	MenuItems[#MenuItems].frame:HookScript("OnEnter", function(self)
		self:SetIcon("ReloadPowerOff")
	end)
	MenuItems[#MenuItems].frame:HookScript("OnLeave", function(self)
		self:SetIcon("Reload")
	end)

	f.DividerV:SetVertexColor(Settings.color.r, Settings.color.g, Settings.color.b, 0.6)
	f.Banner:Show()
	f:SetReloadHighlight(initialising)
	f:GetScript("OnShow")(f) 
end

local CharacterSelectFrame
local CharacterList
local Settings
local function SelectCharacterSettings(func, point, relative, topoint, x, y)
	if not CharacterList then
		CharacterList = {}
		local Profile = NS:GetOption("Profile")
		for k, v in pairs(Settings) do
			if k ~= Profile then
				tinsert(CharacterList, { name=k, entry=v })
			end
		end
		sort(CharacterList, function(a, b)
			return(a.name < b.name)
		end)
	end
	local f
	if CharacterSelectFrame then
		f = CharacterSelectFrame
	else
		local Selected = { name="", entry=nil }
		CharacterSelectFrame = NS.Lib:CreateDialogueBackdrop("Copy", FauxMazzleOptions.OptionSpace, "Frame", true)
		
		CharacterSelectFrame:Hide()
		CharacterSelectFrame:SetBackdropColor(0.06, 0.06, 0.06, 0.8)
		CharacterSelectFrame:SetBackdropBorderColor(0.06, 0.06, 0.06, 0.8)
		f = CharacterSelectFrame
--		f.SelecteName = NS.Lib:CreateFont("Name", f, L.Characters, "CENTER", "CENTER", NS.Font, NS.FontSize-2)
		f.SelecteName = NS.Lib:CreateFont("Name", f, L.Characters, "CENTER", "MIDDLE", NS.Font, NS.FontSize-2)
		f.SelecteName:SetHeight(11)
		f.SelecteName:SetTextColor(1, 0.82, 0)
		f.SelecteName:SetPoint("TOPLEFT", 0, -5)
		f.SelecteName:SetPoint("TOPRIGHT", 0, -5)
		f:SetSize(180, 190)
		f.Characters = NS.Lib:CreateScrollList("Characters", f, Selected, "name", CharacterList, "name", "name", 9)

		f.Characters:SetPoint("TOPLEFT", 5, -21)
		f.Characters:SetPoint("BOTTOMRIGHT", -24, 25)
		f.Characters:SetFont(NS.Font, NS.FontSize-2, "")
		function f.Characters:PostUpdate()
			Selected.entry = self.Table[self.Key].entry
			self:GetParent().Select:SetEnabled(true)
			self:GetParent().Select.Text:SetText(L.Copy..": |cffffd100"..Selected.name.."|r")
		end
		local ButtonSettings = { id="CharacterSelect", nolocale=false, text="Copy", tip=nil, icon="CharCopy"}
		f.Select = NS:CreateMenuButton("Select", f, "Button", ButtonSettings, 25, true)
		f.Select:SetFont(NS.Font, NS.FontSize-1, "")
		f.Select:SetSize(80, 20)
		f.Select:SetPoint("BOTTOMLEFT", 2, 3)
		f.Select:SetPoint("BOTTOMRIGHT", -2, 3)
		f.Select:SetEnabled(false)
		f.Select:HookScript("OnClick", function(self)
			self:GetParent():Hide()
			self.func(Selected.name, Selected.entry)
		end)
	end
	if f:IsShown() then
		f:Hide()
		return
	end
	f.Select.func = func
	f:ClearAllPoints()
	f:SetPoint(point, relative, topoint, x, y)
	f:SetParent(relative)
	f:SetFrameStrata(relative:GetFrameStrata())
	f:SetFrameLevel(relative:GetFrameLevel() + 4)
	f:Show()
end

function NS:PlaceCopyButton(func, xoffset, yoffset, point, relative, topoint, x, y, parent)
-- Only displays if at least one other character has settings to copy
	if not CopyButton then
		Settings = NS:GetSettings("Profiles")
		local Profile = NS:GetOption("Profile")
		for k,_ in pairs(Settings) do
			if k ~= Profile then
				CanCopy = true
				break
			end
		end
		local ButtonSettings = { id="CharacterCopy", nolocale=true, text=L.GetText(L.Settings, L.Copy), tip=nil, icon="Copy"}
		CopyButton = NS:CreateMenuButton("Copy", FauxMazzleOptions.OptionSpace, "Button", ButtonSettings, 20, true)
		CopyButton:SetSize(95, 16)
		CopyButton:HookScript("OnClick", function(self)
			SelectCharacterSettings(self.func, "TOPLEFT", self, "TOPRIGHT", self.xoffset, self.yoffset)
		end)
	end
	if CharacterSelectFrame then
		CharacterSelectFrame:Hide()
	end
	CopyButton.xoffset = xoffset
	CopyButton.yoffset = yoffset
	CopyButton.func = func
	CopyButton:ClearAllPoints()
	CopyButton:SetPoint(point, relative, topoint, x, y)
	if not HookedParents[relative] then
		HookedParents[relative] = true
		relative:HookScript("OnHide", function(self)
			if CharacterSelectFrame then
				CharacterSelectFrame:Hide()
			end
		end)
	end
	CopyButton:SetParent(parent or relative)
	CopyButton:SetFrameStrata(relative:GetFrameStrata())
	CopyButton:SetFrameLevel(relative:GetFrameLevel()+2)
	if not CanCopy then
		CopyButton:Hide()
	else
		CopyButton:Show()
	end
	return CopyButton
end

function NS:CharacterSettings(set, charsettings, branch)
	local SrcSettings = charsettings
	if type(set) == "table" then
		SrcSettings = SrcSettings[upper(set.table)]
		local i = 1
		while i>0 do
			if not set[i] then break end
			SrcSettings = SrcSettings[set[i]]
			i = i+1
		end
	else
		SrcSettings = SrcSettings[upper(branch)]
		if type(set) == "string" then
			SrcSettings = SrcSettings[set]
		end
	end
	return SrcSettings
end

local function ConfirmAction(self)
	local Parent = self:GetParent()
	if Parent.func then
		Parent.func(self.Execute, Parent.menubutton)
	end
	if Parent.menubutton then
		Parent.menubutton:SetChecked(false)
		Parent.menubutton:GetScript("OnClick")(Parent.menubutton)
	end
	OnMenuClick(Parent.menubutton)

	if SettingsPanel then
	 	HideUIPanel(SettingsPanel)
	 	HideUIPanel(GameMenuFrame)
		OptionsFrame:Show()
	end
end
local Points = {
	TOPLEFT="BOTTOMLEFT",
	TOPRIGHT="BOTTOMRIGHT",
	TOP="BOTOM",
}
function NS:ConfirmAction(parent, menubutton, text, func, yestext, notext)
	if not ConfirmDialog then
		ConfirmDialog = CreateFrame("Frame", "$parentConfirmation", parent)
		local f = ConfirmDialog
		f:Hide()
		f.ControlList = {}
		f.OffsetList = {}
		f:SetSize(ButtonWidth + 40, parent.Menu.MinHeight)
		f.Text = NS.Lib:CreateFont("Txt", f, "", "CENTER", "TOP", nil, 14)
		f.Text:SetSize(ButtonWidth+10, 50)
		f.Text:SetPoint("TOP", 0, -20)
		local ButtonSettings = { id="ConfirmYes", nolocale=true, text="", tip=nil, icon="Yes"}
		f.OK = NS:CreateMenuButton("ConfirmYes", f, "Button", ButtonSettings, 20, true)
		f.OK.Execute = true
		f.OK:SetSize(ButtonWidth / 1.6, ButtonHeight)
		f.OK:SetPoint("TOPLEFT", f, "BOTTOM", 0, -15)
		f.OK:SetScript("OnClick", ConfirmAction)
	
		ButtonSettings = { id="ConfirmNo", nolocale=true, text="", tip=nil, icon="No"}
		f.Cancel = NS:CreateMenuButton("ConfirmNo", f, "Button", ButtonSettings, 20, true)
		f.Cancel:SetSize(ButtonWidth / 1.6, ButtonHeight)
		f.Cancel:SetPoint("TOPRIGHT", f.Text, "BOTTOM", 0, -15)
		f.Cancel:SetScript("OnClick", ConfirmAction)
		function f:SetButtonPos(menubutton, yos)
			self.OffsetList[menubutton] = yos
		end
		function f:AddControl(menubutton, control, xos, yos, point)
			if not self.ControlList[menubutton] then
				self.ControlList[menubutton] = {}
			end
			tinsert(self.ControlList[menubutton], control)
			control:ClearAllPoints()
			control:SetPoint(point or "TOPLEFT", self.Text, point and Points[point] or "BOTTOMLEFT", xos, yos)
			control:Hide()
			return #self.ControlList[menubutton]
		end
		function f:ShowControls(menubutton)
			for button, controlist in pairs(self.ControlList) do
				for i=1, #controlist do
					if button == menubutton then
						controlist[i]:Show()
					else
						controlist[i]:Hide()
					end
				end
			end
			local yos = self.OffsetList[menubutton] or -15
			self.OK:ClearAllPoints()
			self.Cancel:ClearAllPoints()
			self.OK:SetPoint("TOPLEFT", self.Text, "BOTTOMLEFT", -10, yos)
			self.Cancel:SetPoint("TOPRIGHT", f.Text, "BOTTOMRIGHT", 10, yos)
		end
	end
	ConfirmDialog:ShowControls(menubutton)
	ConfirmDialog.Text:SetText(text)
	ConfirmDialog.OK:SetText(yestext or L.Yes)
	ConfirmDialog.Cancel:SetText(notext or L.No)
	ConfirmDialog.func = func
	ConfirmDialog.menubutton = menubutton
	ConfirmDialog:Show()
	parent.Options = ConfirmDialog
	return ConfirmDialog
end

function NS:CopySettings(character, charsettings, branch, Controls, CheckButtons)
	local SrcSettings
	if Controls then
		for k, set in pairs(Controls) do
			SrcSettings = NS:CharacterSettings(set, charsettings, branch)
			k:Init(SrcSettings[k.Index])
			if k.PostUpdate then
				k:PostUpdate()
			end
		end
	end
	if CheckButtons then
		for k, set in pairs(CheckButtons) do
			SrcSettings = NS:CharacterSettings(set, charsettings, branch)
			k:Init(SrcSettings[k.Index] or false)
			k:GetScript("OnClick")(k)
		end
	end
end

